#!/usr/bin/python
# -*- coding: utf-8 -*-
#######################################################################################################
#Autor:         J. Felix Ontañón <josef.ontanon.ext@juntadeandalucia.es>
#Fecha:         2006/10/11
#Versión:       0.1
#Licencia:      GNU GPL (http://www.gnu.org/licenses/gpl.txt)
#Descripcion:   Navega automaticamente por la web de JClic en busca de aplicaciones en castellano y las
#		descarga en /publico/Aplicaciones JClic (directorio de descarga por defecto) con capacidad
#		de resumir las descargas.
#
#Uso:           guadalinfo_jclic.py area[,area,area...] donde area es una o varias de las siguientes:
#			Lengua, Matematicas, CienciasSociales, CienciasExperimentales, Musica, PlasticayVisual, 
#			EducacionFisica, Tecnologia, Diversos.
#######################################################################################################

import re
import urllib2
import sys
import os
import os.path
import time
import exceptions

TIME_AREA = 60
TIME_LINK = 2

DOWNLOAD_DIR = '/publico'

AREAS = {'Lengua':'lleng', 'Matematicas':'mat', 'CienciasSociales':'soc', 'CienciasExperimentales':'exp', 'Musica':'mus', 'PlasticayVisual':' vip', 'EducacionFisica':'ef', 'Tecnologia':'tec', 'Diversos':'div'}
NIVELES = {'Infantil':'INF','Primaria':'PRI','Secundaria':'SEC','Bachillerato':'BTX'}
BASE_URL = 'http://clic.xtec.es/db/listact_es.jsp?lang=es&ordre=0&desc=1&from=1&area={AREA}&idioma=es&nivell={NIVEL}&text_titol=&text_aut=&text_desc=&num=1000'
ACTIVITY_LINK = ['http://clic.xtec.es/db/','act_es.jsp?id=']

search_link = re.compile(r'''http://clic.xtec.net/projects/(.|\d)*/jclic/(.|\d)*zip''', re.VERBOSE)
search_lang = re.compile(r'''Versi.{1}n JClic - espa.{1}ol''')
	
class PublicDirectoryError(exceptions.Exception):
	def __init__(self, arg):
		self.message = arg

def create_dirs(areas):
	try:
		os.chdir(DOWNLOAD_DIR)
	except:
		raise PublicDirectoryError('No se tiene acceso al directorio %s' % DOWNLOAD_DIR)

	if not os.path.exists('Aplicaciones JClic'):
		try:
			os.mkdir('Aplicaciones JClic')
		except:
			raise PublicDirectoryError('No se pudo crear el directrio %s/Aplicaciones JClic' % DOWNLOAD_DIR)
		
	os.chdir('Aplicaciones JClic')

	for area in areas.keys():
		if not os.path.exists(area):
			os.mkdir(area)
			
		os.chdir(area)
		
		for nivel in NIVELES.keys():
			if not os.path.exists(nivel):
				os.mkdir(nivel)
		os.chdir('../')


def get_app_links(web_content):
	links_list = []

	start_from_here = 0
	while True:
		index = web_content.find(ACTIVITY_LINK[1], start_from_here)

		if index == -1:
			break
		else:
			link = ACTIVITY_LINK[0]+web_content[index:index+len(ACTIVITY_LINK[1])+4]
			links_list.append(link)
			start_from_here = index+len(ACTIVITY_LINK[1])+4
	
	return links_list


def get_apps_url(link):
	apps_url = []
	app_web = urllib2.urlopen(link)
	content = app_web.read()
	index_espanol = False	

	try:
		for item in search_lang.finditer(content):
			index_espanol = item.start()
			break
			

		if index_espanol:
			for item in search_link.finditer(content, index_espanol):
				app_url = content[item.start():item.end()]
				apps_url.append(app_url)
				break
	except:
		pass

	return apps_url


def load_downloaded_apps():
	print 'Buscando actividades ya descargadas'

	apps_dict = {}
	
	for root, dirs, files in os.walk(DOWNLOAD_DIR+os.sep+'Aplicaciones JClic'):
		for file in files:
			root_dir = root.split('/')
			apps_dict[file]=root_dir[-2]+os.sep+root_dir[-1]

	print 'Se encontraron %d actividades descargadas.' % len(apps_dict)
	return apps_dict
	
	
def main(areas=AREAS):
	try:
		create_dirs(areas)
	except PublicDirectoryError, e:
		os.system('zenity --info --text "%s"' % e.message)
		os.system('zenity --error --text "No se pudo realizar la descarga de aplicaciones"')
		return False

	apps_dict = load_downloaded_apps()
	
	print 'Comenzando la descarga de actividades'
	for area in areas.keys():
		for nivel in NIVELES.keys():
			url = BASE_URL.replace('{AREA}',areas[area]).replace('{NIVEL}',NIVELES[nivel])
			web = urllib2.urlopen(url)
			links_list = get_app_links(web.read())
			web.close()
			
	
			for link in links_list:
				apps_url = get_apps_url(link)
				pair = True
				for app_url in apps_url:
					download_dir = area+os.sep+nivel
					app_file = app_url.split('/')[-1]
					if pair:
						if not apps_dict.has_key(app_file):
							print 'Esperando %d segundos para comenzar la siguiente descarga' % TIME_LINK
							time.sleep(TIME_LINK)
							os.system('wget -c  %s -O %s' % (app_url, download_dir+os.sep+app_file))
							#print app_url, download_dir+os.sep+app_file
							apps_dict[app_file]=download_dir
						else:
							if not os.path.exists(download_dir+os.sep+app_file):
								#print '../../'+apps_dict[app_file]+os.sep+app_file, download_dir+os.sep+app_file
								os.symlink('../../'+apps_dict[app_file]+os.sep+app_file, download_dir+os.sep+app_file)
					pair = not pair
		
		if not len(areas)<=1:
			print 'Se concluyo la descarga del area: %s' % area
			print 'Esperando %d segundos para comenzar la descarga de la siguiente area' % TIME_AREA
			time.sleep(TIME_AREA)
	return True
			

if __name__ == '__main__':
	areas = {}

	if len(sys.argv) > 1:
		for arg in sys.argv[1:]:
			if AREAS.has_key(arg):
				areas[arg]=AREAS[arg]
			else:
				print '%s no es un area valida' % arg
				sys.exit(1)
				
		if main(areas):
			os.system('zenity --info --text "Se concluyo satisfactoriamente la descarga de las aplicaciones"')
		else:
			os.system('zenity --info --text "No se pudo concluir la descarga de las aplicaciones"')
						

	else:
		hlp = 	'Uso: guadalinfo_jclic.py area[,area,area...]\n'
		hlp +=	'Areas:\n'
		for area in AREAS.keys():
			hlp += '	%s\n' % area

		print hlp
